/*
** config.c - handles reading the config file
**
** Keith Zaback (krz@wcug.wwu.edu) Apr 98
*/

/* includes */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ipxfury.h"

/* defines */
#define FURY_CFG_LINE_SIZE	256

/* protos */
void			cleanline(char *line);
int			isblank(char *line);
int			config_interface(char *cmdline, char *filename, int lineno);
int			config_ports(int set, char *cmdline, char *filename, int lineno);
int			config_option(char *args, char *filename, int line);
void			iface_option(int option, char *name, char *filename, int line);

/* externs */
extern GLOBALS	ifg;


/*
**	IFACE_OPTION - sets interface options
*/
void iface_option(int option, char *name, char *filename, int line)
{
	INTERFACE	*iface;
	
	/* did they give an iface name? */
	if( name == NULL ) {
		notice("(%s:%d) Need to specify an interface\n", filename, line);
		return;
	}
	
	/* get interface */
	iface = interface_find_name(name);
	if( NULL == iface ) {
		notice("(%s:%d) Could not find interface '%s'\n", filename, line, name);
		return;
	}
	
	/* set the option */
	iface->options |= option;
	
	/* ok */
	return;
}	
	


/*
**	CONFIG_OPTION
*/
int config_option(char *args, char *filename, int line)
{
	char		*token[5];
	int		num_args;
	INTERFACE	*iface;
	int		ms;
	
	/* do we have any option? */
	if( args == NULL ) {
		notice("(%s:%d) Need to specify an option!\n", filename, line);
		return(0);
	}
		
	/* tokenize the line */
	num_args = tokenize(args, 4, ' ', token);
	
	/* what option do they want? */
	if( strcasecmp(token[0], "forward") == 0 ) {
		iface_option(FOPT_FORWARD, token[1], filename, line);
	} else if( strcasecmp(token[0], "fullframe") == 0 ) {
		iface_option(FOPT_FULLFRAME, token[1], filename, line);		
	} else if( strcasecmp(token[0], "sc") == 0 ) {
		iface_option(FOPT_SC_RESEND, token[1], filename, line);
	} else if( strcasecmp(token[0], "threshold") == 0 ) {
		if( num_args == 4 ) {
			/* get the udp interface! */
			iface = interface_find_name("udp");
			if( NULL != iface ) {
				/* set threshold options */
				iface->data.u.u_buffer_threshold = strtol(token[1], NULL, 0);
				ms = strtol(token[2], NULL, 0);
				iface->data.u.u_buffer_wait_sec = ms / 1000;
				iface->data.u.u_buffer_wait_usec = (ms % 1000) * 1000;
				iface->generation_max		= strtol(token[3], NULL, 0);
			} else {
				notice("(%s:%d) Threshold option only applies to the udp interface!\n", filename, line);		
			}
		} else {
			notice("(%s:%d) Need 3 arguments after option 'threshold'\n", filename, line);
			return(0);
		}
			
	} else {
		notice("(%s:%d) Unknown option '%s'\n", token[0]);
		return(0);
	}	
	
	/* did ok */
	return(1);

}



/*
**	CONFIG_PORTS - handles "watchport" and "ignoreport" commands
**
**	The format:
**		watchport [all | number]
**		ignoreport [all | number]
*/
int config_ports(int set, char *cmdline, char *filename, int lineno)
{
	int		port_number;
	char		*token[2];

	/* give them a message if they give us bad args */
	if( cmdline == NULL ) {
		notice("(%s:%d) need some arguments for ", filename, lineno);
		if( set )
			notice("watchport!\n");
		else
			notice("ignoreport!\n");
		return(0);
	}
	
	/* for everything they put on the cmd line */
	do {
		tokenize(cmdline, 2, ' ', token);
		cmdline = token[1];
		if( token[0] != NULL ) {
			if( strcasecmp(token[0], "all") == 0 ) {
				/* they want them all */
				if( set ) 
					set_all_ports(ifg.watched_ports);
				else
					unset_all_ports(ifg.watched_ports);
			} else {
				/* assume number */
				port_number = strtol(token[0], NULL, 0);
				if( (port_number < 0) || (port_number > 65535) ) {
					/* bad port */
					notice("(%s:%d) port number invalid (%d)!\n", filename, lineno, port_number);
				} else {
					/* good port */
					if( set )
						set_port(ifg.watched_ports, port_number);
					else
						unset_port(ifg.watched_ports, port_number);
				} /* good port */
			} /* not "all" */
		} /* good arg */
		
	} while( token[0] != NULL );
	
	/* done */
	return(1);
}


/*
**	CONFIG_INTERFACE - handles the "interface" config command
**
**	The format is:
**		interface [serial | modem] name baud
**		interface ether name frametype
**		interface udp port 
**
*/
int config_interface(char *cmdline, char *filename, int lineno)
{
	char		*arg[4];
	INTERFACE	*iface;
	
	/* have we passed the maximum interfaces? */
	if( ifg.total_interfaces >= FURY_MAX_INTERFACES ) {
		errormsg(0, "config_interface", "(%s:%d) only %d interfaces are allowed!", filename, lineno, FURY_MAX_INTERFACES);
		return(0);
	}
	
	/* if not, set a pointer to an interface struct to work with */
	iface = &ifg.interfaces[ifg.total_interfaces];
	
	/* clean it out */
	memset(iface, 0, sizeof(INTERFACE));
	
	/* get all the args */
	tokenize(cmdline, 3, ' ', arg);
	
	/* valid args? */
	if( arg[0] == NULL ) {
		errormsg(0, "config_interface", "(%s:%d) not enough arguments for 'interface' cmd", filename, lineno);
		return(0);
	}
			
	/* what type of device it is, "serial" or "ether" */
	if( strcasecmp(arg[0], "serial") == 0 ) {
		iface->type = FURY_IF_TYPE_SERIAL;
#ifdef FURY_ETHERNET
	} else if( strcasecmp(arg[0], "ether") == 0 ) {
		iface->type = FURY_IF_TYPE_ETHER;
#endif
	} else if( strcasecmp(arg[0], "modem") == 0 ) {
		iface->type = FURY_IF_TYPE_SERIAL;
		iface->flags |= FURY_IF_MODEM_LINES;	/* use modem control */
	} else if( strcasecmp(arg[0], "udp") == 0 ) {
		iface->type = FURY_IF_TYPE_UDP;
		
		/* set up some defaults */
		iface->data.u.u_buffer_threshold = FURY_UDP_BUFFER_THRESHOLD; 
		iface->data.u.u_buffer_wait_sec	 = FURY_UDP_BUFFER_WAIT_SEC;
		iface->data.u.u_buffer_wait_usec = FURY_UDP_BUFFER_WAIT_USEC;
	} else {
		errormsg(0, "config_interface", "(%s:%d) unknown device type '%s'", filename, lineno, arg[1]);
		return(0);
	}
	
	/* the device name -- if it is not UDP */
	if( iface->type != FURY_IF_TYPE_UDP ) {
		/* check args */
		if( arg[1] == NULL ) {
			errormsg(0, "config_interface", "(%s:%d) no device name!", filename, lineno);
			return(0);
		}
		strncpy(iface->name, arg[1], FURY_IF_NAME_MAX-1);
		iface->name[FURY_IF_NAME_MAX-1] = 0x00;
	} else {
		/* put in something */
		strcpy(iface->name, "udp");
	}
		
	/* either baudrate or frametype */
	if( iface->type == FURY_IF_TYPE_SERIAL ) {
		if( arg[2] != NULL ) {
			iface->data.s.s_baudrate = strtol(arg[2], NULL, 0);
		} else {
			errormsg(0, "config_interface", "(%s:%d) no baudrate!", filename, lineno);
			return(0);
		}
	} else if( iface->type == FURY_IF_TYPE_ETHER ) {
		if( arg[2] != NULL ) {
			if( strcasecmp(arg[2], "ii") == 0 ) {
				iface->data.e.e_frametype = FURY_FRAMETYPE_II;
			} else if( strcasecmp(arg[2], "802.3") == 0 ) {
				iface->data.e.e_frametype = FURY_FRAMETYPE_802_3;
			} else if( strcasecmp(arg[2], "802.2") == 0 ) {
				iface->data.e.e_frametype = FURY_FRAMETYPE_802_2;
			} else {
				errormsg(0, "config_interface", "(%s:%d) unknown frametype '%s'", filename, lineno, arg[3]);
				return(0);
			}
		} else {
			errormsg(0, "config_interface", "(%s:%d) no frametype!", filename, lineno);
			return(0);
		}
	} else if( iface->type == FURY_IF_TYPE_UDP ) {
		if( arg[1] != NULL ) {
			/* port number */
			iface->data.u.u_portnumber = strtol(arg[1], NULL, 0);
		} else {
			errormsg(0, "config_interface", "(%s:%d) no udp port number!", filename, lineno);
			return(0);
		}	
	}

	/* set the options */
	iface->options		= FOPT_NONE;
	
	/* set generation stuff */
	iface->generation	= 0;
	iface->generation_max	= FURY_DIFF_GENERATION_MAX;
	
	/* did ok! */
	ifg.total_interfaces++;
	
	return(1);	

}



/*
**	READ_CONFIG()
*/
int read_config(char *filename, CONFIG *cfg)
{
	FILE		*cfgfile;
	char		line[FURY_CFG_LINE_SIZE];
	int		linenumber;
	char		*token[2];

	/* now try to the file they want */
	cfgfile = fopen(filename, "r");
	if( cfgfile == NULL ) {
		errormsg(0, "read_config", "could not open config file '%s'", filename);
		return(0);
	}
	
	/* start reading each line */
	linenumber = 1;
	while( !feof(cfgfile) ) {
		line[0] = 0x00;
		fgets(line, (FURY_CFG_LINE_SIZE-1), cfgfile);	/* get the line */
		cleanline(line);		/* remove chars we don't want */
		if( ! isblank(line) ) {		/* process it if it not blank */
			/* get 1st word (command) */
			tokenize(line, 2, ' ', token);
			
			/* process the command */
			if( strcasecmp(token[0], "interface") == 0 ) {
				config_interface(token[1], filename, linenumber);
			} else if( strcasecmp(token[0], "watchport") == 0 ) {
				config_ports(1, token[1], filename, linenumber);
			} else if( strcasecmp(token[0], "ignoreport") == 0 ) {
				config_ports(0, token[1], filename, linenumber);
			} else if( strcasecmp(token[0], "option") == 0 ) {
				config_option(token[1], filename, linenumber);
			} else {
				errormsg(0, "read_config", "(%s:%d) unknown option '%s'", filename, linenumber, token[0]);
			}
		} /* non-blank */
		
		/* increment the line counter */
		linenumber++;
		
	} /* while */
	
	/* close the file */
	fclose(cfgfile);
	
	/* did ok */
	return(1);

}



/*
**	CLEANLINE()
*/
void cleanline(char *line)
{
	char		*src, *dest;
	char		templine[FURY_CFG_LINE_SIZE];
	char		lastchar;
	
	/* set our src and dest pointers */
	src = line;
	dest = templine;
	
	/* skip leading white space */
	while( (*src == ' ') || (*src == '\t') )
		src++;
	
	/* now loop */
	lastchar = 0x00;
	while( *src != 0x00 ) {
		switch( *src ) {
			case '\t':
				*src-- = ' ';	/* convert to space */
				break;
			case '\r':		/* do not want these */
			case '\n':
				break;
			default:
				/* only copy at max, 1 space */
				if( lastchar == ' ' ) {
					if( *src != ' ' )
						*dest++ = *src;
				} else {
					*dest++ = *src;
				}
				break;
					
		}
		/* adjust last char */
		lastchar = *src;
		
		/* inc src ptr */
		src++;
	}
	
	/* check for ending spaces */
	if( dest > templine ) {
		dest--;
		if( *dest == ' ' )
			*dest = 0x00;
		dest++;
	}
	
	/* NULL end the string */
	*dest = 0x00;
	
	/* now replace the string */
	strcpy(line, templine);
	
	/* done */
	return;
}



/*
**	ISBLANK() - returns 1 if line is blank or a comment
*/
int isblank(char *line)
{
	int		blank;
	
	if( *line == '#' )
		return(1);		/* a comment */
		
	blank = 1;			/* set blank to true */
	while( (blank) && (*line != 0x00) ) {
		if( *line != ' ' )	/* not a space, then it is non-blank */
			blank = 0;
		line++;
	}
	
	/* return our result */
	return(blank);
}



