/*
** console.c - the fury console
**
** Keith Zaback (krz@wcug.wwu.edu) Oct 98
*/

#include "ipxfury.h"
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#ifdef CURSES_CONSOLE

#ifdef	FURY_LINUX
#include <curses.h>
#endif

#ifdef	FURY_BSD
#include <ncurses.h>
#endif

WINDOW		*main_win;
WINDOW		*status_bar;
WINDOW		*cmd_line;

void		console_redraw(void);

#else

struct termios	tio_save;

#endif /* no-curses */



/* globals */
extern GLOBALS	ifg;
char		command_buffer[FURY_COMMAND_BUF_SIZE];
int		command_index;


/*
**	TERMINAL_READ() - read stuff from the terminal (curses and non)
**
**      Note: this function returns 1 on ^D
*/
int terminal_read(int fd)
{

#ifdef CURSES_CONSOLE
	int		ch;
	
	ch = wgetch(cmd_line);
	
#else
	int		ch;
	char		c;
	
	/* read 1 char -- sorta crappy but oh well */
	if( read(fd, &c, 1) != 1 )
		return(0);
		
	/* make it an int */
	ch = c;
#endif

	switch( ch ) {
		case 0x04:	/* ^D */
			if( ifg.status == FSTATUS_FURIOUS ) {
				notice("Attempting to close down the interfaces...\n");
				return(1);
			}
			break;
		
		case 0x03:	/* ^C */
			if( ifg.status == FSTATUS_FURIOUS ) {
				notice("Restraining the FURY.\n");
				return(2);
			}
			break;
			
		case 0x01:	/* ^A */
			fprintf(ifg.logfile, "************** MARK ******************\n");
			notice("Marked!\n");
			break;
			
		case 0x06:	/* ^F */
			fflush(ifg.logfile);
			notice("Logfile flushed!\n");
			break;
#ifdef CURSES_CONSOLE

		case 0x0c:		/* ^L */
			/* touch & refresh */
			console_redraw();
			break;
	
		case '\n':
		case '\r':
			/* NULL end buffer */
			command_buffer[command_index] = 0x00;
			command_index = 0x00;
			/* clean line */
			wmove(cmd_line, 0, 0);
			wclrtoeol(cmd_line);
			wrefresh(cmd_line);
			
			/* process.. */
			process_terminal_command(command_buffer);
			break;
			
		case KEY_BACKSPACE:
			if( command_index > 0 ) {
				waddstr(cmd_line, "\010 \010");
				command_index--;
			} else {
				beep();
			}
			wrefresh(cmd_line);
			break;					
				
		default:
			if( command_index < (FURY_COMMAND_BUF_SIZE-1) ) {
				/* add it in */
				command_buffer[command_index++] = ch;
				waddch(cmd_line, ch);
			} else {
				/* too full */
				beep();
			}
			wrefresh(cmd_line);
			break;			
		
#else			
			
		case '\n':	
		case '\r':
			putchar('\n');
			command_buffer[command_index] = 0x00;
			command_index = 0;
			process_terminal_command(command_buffer);	/* do the command */
			if( (ifg.status == FSTATUS_PROMPT) && (! ifg.done) ) {
				printf("IPXFury> ");
			}
			break;
			
		default:
			if( ch == tio_save.c_cc[VERASE] ) {		/* BACKSPACE */
				if( command_index > 0 ) {
					command_index--;
					command_buffer[command_index] = 0x00;
					printf("%c %c", 8, 8);
				}		
			} else if( ch >= 0x20 ) {			/* printable only */
				if( command_index < (FURY_COMMAND_BUF_SIZE-1) ) {
					command_buffer[command_index] = ch;
					command_index++;
					command_buffer[command_index] = 0x00;
					putchar(ch);		/* echo it now */
				}
			}
#endif
			
	} /* switch */
	

	/* done */
	return(0);
}




/* should we curse it up */
#ifdef CURSES_CONSOLE

/********************************************************************
*****			CURSES CONSOLE CODE			*****
********************************************************************/


/*
**	CONSOLE_REDRAW
*/
void console_redraw()
{
	/* clear them all */
	wclear(main_win);
	wclear(status_bar);
	wclear(cmd_line);
	
	/* refresh main */
	wrefresh(main_win);
	
	/* draw status */
	draw_statusbar(NULL);
	wrefresh(status_bar);
	
	/* now the command line */
	command_buffer[command_index] = 0x00;
	mvwaddstr(cmd_line, 0, 0, command_buffer);
	wrefresh(cmd_line);
	
	/* done */
	return;
}

/*
**	INIT_CONSOLE - curses windows
*/
int init_console(int *fd)
{
	int		x, y;
	char		*my_tty;
	int		tty;

	/* get our tty */  
	my_tty = ttyname(fileno(stdin));
	
	/* open up terminal */
	if( (tty = open(my_tty, O_RDWR)) < 0 ) {
		printf("[init_console] could not open %s\n", my_tty);
		return(0);
	}
	
	/* save it */
	*fd = tty;

	/* startup */
	if( NULL == initscr() ) {
		close(tty);
		return(0);
	}
	
	/* set our modes */
	cbreak();
	noecho();
	nonl();
	
	/* get size of screen */
	getmaxyx(stdscr, y, x);
	
	/* init the ptrs */
	main_win	= NULL;
	status_bar	= NULL;
	cmd_line	= NULL;
	
	/* create the main window */
	main_win = newwin(y-2, x, 0, 0);
	if( NULL == main_win ) {
		printf("Failed to make main window\n");
		close(tty);
		return(0);
	}
	
	/* set options */
	scrollok(main_win, TRUE);
	wsetscrreg(main_win, 0, y-3);
	
	/* create the status bar */
	status_bar = newwin(1, x, y-2, 0);
	if( NULL == status_bar ) {
		printf("Failed to make status bar\n");
		close(tty);
		return(0);
	}
	
	/* set options */
	wbkgd(status_bar, ' ' | A_REVERSE);
	
	/* create the cmd line */
	cmd_line = newwin(1, x, y-1, 0);
	if( NULL == cmd_line ) {
		printf("Failed to make command line\n");
		close(tty);
		return(0);
	}
	
	/* keypad style */ 
	keypad(cmd_line, TRUE);
	
	/* get them all */
	wrefresh(main_win);
	wrefresh(status_bar);
	wrefresh(cmd_line);
	
	/* ok */
	return(1);
}


/*
**	EXIT_CONSOLE - curses style 
*/
void exit_console(int fd)
{

	/* close the tty */
	close(fd);
	
	/* kill the windows */
	if( main_win != NULL )
		delwin(main_win);
	if( status_bar != NULL )
		delwin(status_bar);
	if( cmd_line != NULL )
		delwin(cmd_line);
		
	/* we are done */
	endwin();
		
	/* ok */
	return;
}


/*
**	DRAW_STATUSBAR - cool status bar thingy
*/
void draw_statusbar(char *text)
{
	int		x, y;
	static char	status_text[128];
	
	/* copy text if we need it */
	if( text != NULL ) {
		strncpy(status_text, text, 127);
		status_text[127] = 0x00;
	}
	
	/* go away if background */
	if( ifg.config.background ) {
		return;
	}
	
	/* get status bar size */
	getmaxyx(status_bar, x, y);
	
	/* adjust y for our banner thingy */
	y -= 16;
	
	/* do we have enough? */
	if( y <= 0 ) {
		y = 0;
	}	

	status_text[y] = 0x00;
	
	/* draw it */
	wmove(status_bar, 0, 0);
	wclrtoeol(status_bar);
	waddstr(status_bar, " -=[ IPXFURY ]=-");
	mvwaddstr(status_bar, 0, 20, status_text);
	
	/* refresh it */
	wrefresh(status_bar);
	wrefresh(cmd_line);
		
	/* done */
}




#else





/********************************************************************
****		NON CURSES CONSOLE CODE				*****
********************************************************************/

/*
**	EXIT_CONSOLE - non curses..
*/
void exit_console(int tty)
{
	/* return the attributes to normal */
	tcsetattr(tty, TCSANOW, &tio_save);
		
	/* close the tty */
	close(tty);
	
	/* ok */
	return;
	
}



/*
**	INIT_CONSOLE - non curses...
*/
int init_console(int *fd)
{
	struct termios		tio;
	int			tty;
	char			*my_tty;
	
	/* get it */  
	my_tty = ttyname(fileno(stdin));
	
	/* open up terminal */
	if( (tty = open(my_tty, O_RDWR)) < 0 ) {
		printf("[init_console] could not open %s\n", my_tty);
		return(0);
	}
	
	/* get attributes */
	if( tcgetattr(tty, &tio) < 0 ) {
		printf("[init_console] could not get terminal attributes\n");
		close(tty);
		return(0);
	}
	
	/* save a copy of current settings */
	memcpy(&tio_save, &tio, sizeof(tio));
	
	/* set it to raw mode, but with a twist */
	cfmakeraw(&tio);
	tio.c_oflag |= (OPOST | ONLCR);		/* CR = NL-CR */
	
	/* set the attributes */
	if( tcsetattr(tty, TCSANOW, &tio) < 0 ) {
		printf("[init_console] could not set terminal mode\n");
		close(tty);
		return(0);
	}
	
	/* set output buffer to nothing */
	setvbuf(stdout, NULL, _IONBF, 0);
	
	/* set fd to tty and return glorious */
	*fd = tty;
	
	/* start the command buffer */
	command_index = 0;
	memset(command_buffer, 0, FURY_COMMAND_BUF_SIZE);
	
	/* did fine */
	return(1);
	
}



#endif /* no-curses */


