/*
** fury_ipx.c -	the IPX routines to handle IPX data from the fury frame
**
** Keith Zaback (krz@wcug.wwu.edu) Apr 98
*/

/* includes */
#include "ipxfury.h"
#include <string.h>
#include <stdlib.h>
#include <netinet/in.h>

extern GLOBALS	ifg;

/* protos */
int		get_address_locations(INTERFACE *iface, IPXHEADER *ipxheader, HWADDR_ENTRY **src_address, HWADDR_ENTRY **dest_address);

int		process_ipx(INTERFACE *fdev, int type, BYTE *data, int data_length);

int		process_hdrcache_clear(INTERFACE *fdev, BYTE *frame_data, int data_length);
int		process_pktcache_clear(INTERFACE *fdev, BYTE *frame_data, int data_length);



/*
**	CREATE_FURY_IPX_FRAME - make a fury IPX frame
*/
int create_fury_ipx_frame(INTERFACE *fdev, BYTE *orig_ipxframe, BYTE *new_fury_frame, int *new_ftype, int *length)
{
	IPXHEADER	*ipxheader, hdrcache_search;
	int		ipx_frame_length, hdrindex, ipx_data_length;
	HDRCACHE_ENTRY	*hdr_ch_entry;
	BYTE		*ipx_data, *new_fury_data;
	HDRCACHE_INFO	*hdr_ch_info;
	PKTCACHE_ENTRY	pktcache_search, *pktcache_entry;
	int		new_fury_length, ftype;
	IPXHEADER_SQ	ipxheader_sq;


	/* if we could not write everything last time, drop this	*/
	/* on the floor (or in the bit bucket, etc, etc)		*/
	if( fdev->flags & FURY_IF_TOO_MUCH_DATA ) {
		return(0);
	} 

	/* clean the structs */
	memset(&pktcache_search, 0, sizeof(PKTCACHE_ENTRY));
		
	/* fetch the length from the frame */
	ipxheader		= (IPXHEADER *)orig_ipxframe;
	ipx_frame_length	= ntohs(ipxheader->length);
	ipx_data		= (BYTE *)orig_ipxframe + sizeof(IPXHEADER);
	ipx_data_length		= ipx_frame_length - sizeof(IPXHEADER);
	new_fury_length		= 0;
		
	/* make sure it is valid */
	if( (ipx_frame_length < 30) || (ipx_frame_length > FURY_FRAME_MAX_DATA) ) {
		/* frame size error! */
		errormsg(0, "fury_ipx_frame", "bad ipx frame size (%d bytes)", ipx_frame_length);
		return(0);
	}

	/* do we even want to deal with this packet? */
	if( ! check_port(ifg.watched_ports, ntohs(ipxheader->ipx_dest.port)) ) {
		/* -- FIXTHIS : keep track of what ports we have blocked? -- */
		return(0);
	}

	/* build entry to search for in the ipx header cache */	
	memcpy(&hdrcache_search, ipxheader, sizeof(IPXHEADER));
	hdrcache_search.length = 0;		/* always zero */
	hdrcache_search.checksum = 0xFFFF;	/* always 0xFFFF */
	
	/* look	in the ipx header cache for the header */
	hdrindex = hdrcache_lookup(fdev->cache_ptrs.send_hdrcache, &hdrcache_search);
	
	/* if it is not there then we need to add it */
	if( hdrindex == -1 ) {
		fdev->stats.hdrcache_misses++;
		/* insert it */
		hdrindex = hdrcache_insert(fdev->cache_ptrs.send_hdrcache, &hdrcache_search);
		if( hdrindex == -1 ) {
			errormsg(0, "fury_ipx_frame", "could not add ipx header to send cache");
			return(0);
		} 
	} else {
		fdev->stats.hdrcache_hits++;
	}
  	/* get this entry */
	hdr_ch_entry = &fdev->cache_ptrs.send_hdrcache->table[hdrindex];
	
	/* now we have a ptr to a ipx header cache entry! */


	/* -- now we need to find a packet cache entry -- */
	
	/* lookup this source:port address in the packet cache */
	/* build the packet cache search -- copy the net:host:port from the source */
	memcpy(&pktcache_search.address, &ipxheader->ipx_src, sizeof(IPXADDR));
	
	/* next look it up in the send packet cache */
	pktcache_entry = hash_get(fdev->cache_ptrs.send_pktcache, &pktcache_search);
	if( pktcache_entry != NULL ) {
		/* there is an entry for this source:port */
		/* now compare the data			  */
		if( pktcache_entry->data_length == ipx_data_length ) {
			/* actually try a memory compare */
			if( memcmp(pktcache_entry->data, ipx_data, ipx_data_length) == 0 ) {
				/* we are sending the exact same data */
				/* so do nothing... */
				fdev->stats.pktcache_hits++;
			} else {
				/* this data is the same length but different! */
				/* so lets change the data in the cache...     */
				memcpy(pktcache_entry->data, ipx_data, ipx_data_length);
				
				/* reset the used count */
				pktcache_entry->used = 0;
				
				/* stats */
				fdev->stats.pktcache_misses++;
			}
		} else {
			/* length is different, so the data is not the same */
			/* so adjust the entry...                           */
			pktcache_entry->data_length = ipx_data_length;
			memcpy(pktcache_entry->data, ipx_data, ipx_data_length);
			
			/* reset used count */
			pktcache_entry->used = 0;
			
			/* stats */
			fdev->stats.pktcache_misses++;
		}
	} else {
		/* no entry for this source:port in the send packet cache */
		/* lets add an entry...					  */
		pktcache_search.data_length = ipx_data_length;
		memcpy(&pktcache_search.data, ipx_data, ipx_data_length);
		pktcache_entry = hash_insert(fdev->cache_ptrs.send_pktcache, &pktcache_search);
		if( pktcache_entry == NULL ) {
			/* could not insert! */
			errormsg(0, "fury_ipx_frame", "could not add packet cache entry!");
			return(0);
		}
		/* used count will be zero, because of the memset */
		/* at the top of this function			  */
		
		/* stats */
		fdev->stats.pktcache_misses++;
	}
	
	/* -- now we have a pointer to an ipx header cache entry	-- */
	/* -- and a pointer to a packet cache entry			-- */
	/* -- now construct the proper packet for sending		-- */
	
	/* the base ftype is FTYPE_IPX */
	ftype = FTYPE_IPX;
	
	/* deal with the possibly cached header 1st */
	hdr_ch_info = (HDRCACHE_INFO *)new_fury_frame;
	
	/* has this header cache been used before? */
	if( hdr_ch_entry->used_count > 0 ) {
		/* yes, so send a cached header */		
		/* send the header cached ipx frame info block */
		hdr_ch_info->index	= hdrindex;			/* copy index */
		hdr_ch_info->length	= htons(ipx_frame_length);	/* ipx frame length */
		
		/* position the new data pointer destination */
		new_fury_data	= new_fury_frame + sizeof(HDRCACHE_INFO);
		
		/* add our bytes to the new length */
		new_fury_length	+= sizeof(HDRCACHE_INFO);
	} else {
		/* build the header cache info block */			
		hdr_ch_info->index 		= hdrindex;	/* the index */
		hdr_ch_info->length 		= 0xFFFF;	/* this marks an new entry */
		
		/* build a squished ipx header */
		ipxheader_sq.length = htons(ipx_frame_length);	/* the proper length */
		memcpy(ipxheader_sq.src_host, &ipxheader->ipx_src.host, 8);	/* host(6) + port(2) */
		memcpy(ipxheader_sq.dest_host, &ipxheader->ipx_dest.host, 8);	/* same reason */

		/* copy the squished header after the cache info block */
		memcpy(new_fury_frame + sizeof(HDRCACHE_INFO), &ipxheader_sq, sizeof(IPXHEADER_SQ));

		/* position the data pointer */
		new_fury_data	= new_fury_frame + sizeof(HDRCACHE_INFO) + sizeof(IPXHEADER_SQ);
			
		/* adjust the length */
		new_fury_length	+= sizeof(HDRCACHE_INFO) + sizeof(IPXHEADER_SQ);
			
	} /* new header cache entry */
	
	/* -- now we need to add the data part to this new fury	-- */
	/* -- frame.......					-- */

	/* has this packet data cache been used before or too many times? */
	if( (pktcache_entry->used == 0) || (pktcache_entry->used > PKTCACHE_MAX_USE_COUNT) ) {
		/* the other side has not seen this one before.. so send it all */
		memcpy(new_fury_data, ipx_data, ipx_data_length);
		
		/* adjust length */
		new_fury_length	+= ipx_data_length;
		
		/* set the used count to zero in-case of a "too many" timer */
		pktcache_entry->used = 0;	
		
		/* do _NOT_ set the PKT_CACHED mode, since we want to	*/
		/* actually re-send this packet incase something is	*/
		/* wrong with the cache on the other side..		*/
		
	} else {
		/* this is not a 1st timer or "too many" timer so no need to send data */
		ftype |= FTYPE_PKT_CACHED;	/* set packet cashed mode */
		
		/* the ultimate trick */
		/* if we are using a cached header, do not send the length */
		if( hdr_ch_info->length != 0xFFFF ) {
			/* only send _1_ byte.. the cached header index */
			new_fury_frame[0] = hdr_ch_info->index;
			new_fury_length = 1;
			/* set the length cached bit */
			/* ftype |= FTYPE_LENGTH_CACHED; */
			/* deal with the stats */
			ifg.stats.one_byte_packets++;
		}
		/* if not there is nothing to change */
		
	}
	
	/* now increase the used count */
	pktcache_entry->used++;

#if 0
	notice("Built FURY Frame for Sending:\n");
	hexdump(new_fury_frame, new_fury_length);
#endif
	
	/* DONE! -- just copy the length and type */
	*length 	= new_fury_length;
	*new_ftype	= ftype;
	
	/* all done */
	return(1);
	
}


/*
**	PROCESS_IPX - handles IPX data packets (a monster!)
*/
int process_ipx(INTERFACE *fdev, int type, BYTE *data, int data_length)
{
	BYTE		new_ipx_frame[FURY_IPX_FRAME_MAX];
	IPXHEADER	*new_ipx_header;
	BYTE		*new_ipx_data;
	BYTE		*current_src;
	int		bytes_left, new_ipx_length, i;
	HDRCACHE_INFO	hdr_ch_info;
	HDRCACHE_ENTRY	*hdrcache_entry;
	PKTCACHE_ENTRY	pktcache_search, *pktcache_entry;
	HWADDR_ENTRY	*hwaddr_src, *hwaddr_dest;
	IPXHEADER_SQ	*ipxheader_sq;
	int		length_cached;

	/* only do this if we are furious */
	if( ifg.status != FSTATUS_FURIOUS ) {
		return(0);
	}
	
	/* init some variables */
	current_src	= data;			/* where we are currently processing */
	bytes_left	= data_length;		/* how much data we have to work with */
	new_ipx_header	= (IPXHEADER *)new_ipx_frame;		/* our new ipx header */
	new_ipx_data	= new_ipx_frame + sizeof(IPXHEADER);	/* the data location */
	new_ipx_length	= 0;					/* so far, no bytes */
	memset(&pktcache_search, 0, sizeof(PKTCACHE_ENTRY));	/* clean this out */
	
	/* need to construct a "normal" IPX frame, start with the header */

	/* there is _ALWAYS_ header cache info block before a possible header 	*/
	/* with a new header the length == 0xffff	   			*/
	/* _BUT_ if the length cached bit is set then there is only 1 byte	*/

	/* if( FHDR_CACHED_LENGTH(type) ) { */
	if( bytes_left == 1 ) {
		/* only 1 byte follows (the index) */
		hdr_ch_info.index	= *current_src;		/* the only byte */
		hdr_ch_info.length	= 0x0000;		/* temp */	
		current_src++;
		bytes_left--;	
		
		/* set the length cached variable */
		length_cached = 1;
	} else {
		/* a whole cached header info block */
		if( bytes_left < sizeof(HDRCACHE_INFO) ) {
			errormsg(0, "proc_ipx", "no data for header cache info block");
			return(0);
		}
		/* set the header cache info ptr, and adjust the data variables */
		memcpy(&hdr_ch_info, current_src, sizeof(HDRCACHE_INFO));
		current_src	+= sizeof(HDRCACHE_INFO);
		bytes_left	-= sizeof(HDRCACHE_INFO);
		
		/* unset the length cached variable */
		length_cached = 0;
	}
		
	/* make sure the index they are using is valid */
	if( hdr_ch_info.index >= fdev->cache_ptrs.recv_hdrcache->size ) {
		errormsg(0, "proc_ipx", "cache index invalid (%d)", hdr_ch_info.index);
		return(0);
	}
			
	/* index is valid, fetch the entry */
	hdrcache_entry = &fdev->cache_ptrs.recv_hdrcache->table[hdr_ch_info.index];
		
	/* now, does an IPX header follow this or is this a cached entry? */
	if( hdr_ch_info.length == 0xFFFF ) {
		/* yes, a squished IPX header follows.. so we need to cache it */
			
		/* 1st, is there enough data for the actual frame? */
		if( bytes_left < sizeof(IPXHEADER_SQ) ) {
			errormsg(0, "proc_ipx", "no squished ipx header data to copy to cache");
			return(0);
		}
		/* setup a ptr and adjust data vars */
		ipxheader_sq	 = (IPXHEADER_SQ *)current_src;
		current_src	+= sizeof(IPXHEADER_SQ);
		bytes_left	-= sizeof(IPXHEADER_SQ);
		
		/* use this squsihed header to build the new frame */
		memset(new_ipx_header, 0x00, sizeof(IPXHEADER));	/* clean it */
		new_ipx_header->checksum	= 0xFFFF;	/* default */
		new_ipx_header->control		= 0x04;		/* what every other packet uses */
		new_ipx_header->length		= ipxheader_sq->length;	/* both are in NBO */
		memcpy(&new_ipx_header->ipx_src.host, ipxheader_sq->src_host, 8);
		memcpy(&new_ipx_header->ipx_dest.host, ipxheader_sq->dest_host, 8);
		
		/* next setup this cache entry */
		hdrcache_entry->flags		= CACHE_NEW_FLAGS;
		hdrcache_entry->access_remove	= 0;	/* not used */
		hdrcache_entry->used_count	= 1;	/* 1st used */
		memcpy(&hdrcache_entry->ipx_header, new_ipx_header, sizeof(IPXHEADER));
		hdrcache_entry->ipx_header.length = 0;	/* blank this in the cache entry */
			
		/* adjust calculated length */
		new_ipx_length	+= sizeof(IPXHEADER);
			
	} else {
		/* _NO_ squished ipx frame follows the header cache info block */
		/* check the flags to make sure it is a valid entry            */
		if( (hdrcache_entry->flags & CACHE_VALID) != CACHE_VALID ) {
			if(  hdrcache_entry->clear_req <= 0 ) {
				/* set the req */
				errormsg(0, "proc_ipx", "cache entry %d is invalid! (clear sent!)", hdr_ch_info.index);
				send_fury_command(fdev, FCMD_CLEAR_HDRCACHE, (BYTE *)&hdr_ch_info.index, 1);
				/* add to the counter */
				hdrcache_entry->clear_req = HDRCACHE_CLEAR_START;
			} else {
				/* dec the counter */
				hdrcache_entry->clear_req--;			
			}
			/* done */
			return(0);
		}
			
		/* build the header from the cache entry & info */
		memcpy(new_ipx_header, &hdrcache_entry->ipx_header, sizeof(IPXHEADER));
			
		/* increment its use counter */
		hdrcache_entry->used_count++;
			
		/* adjust the length field in the new ipx header  	*/
		/* both of these values are in network byte order 	*/
		/* this value will be wrong with length caching		*/
		/* but will get fixed after the data cache block	*/
		/* has been retreived!					*/
		new_ipx_header->length	= hdr_ch_info.length;
			
		/* adjust the calculated length */
		new_ipx_length	+= sizeof(IPXHEADER);
	}			
		
	/* now we need to add the data to the frame */
	/* lookup the src:port combo in the recv packet cache */
	memcpy(&pktcache_search.address, &new_ipx_header->ipx_src, sizeof(IPXADDR));
		
	/* look for it */
	pktcache_entry = hash_get(fdev->cache_ptrs.recv_pktcache, &pktcache_search);	
	
	if( FHDR_CACHED_DATA(type) ) {
		/* the data has been cached, hopefully we have an entry */
		if( pktcache_entry == NULL ) {
			/* -+- add this entry in, but with invalid data.. */
			/* finish the search structure */
			pktcache_search.data_length	= 0;
			pktcache_search.used 		= 0;
			pktcache_search.clear_req	= 0;
			memcpy(pktcache_search.data, current_src, bytes_left);
			
			/* insert this! */
			pktcache_entry = hash_insert(fdev->cache_ptrs.recv_pktcache, &pktcache_search);
			if( pktcache_entry == NULL ) {
				/* no hope! */
				errormsg(0, "proc_ipx", "could not insert fake packet cache entry");
				return(0);
			}
			
			/* we are screwed! */
			errormsg(0, "proc_ipx", "no cached packet to build ipx frame!");
			/* tell peer that their pkt cache needs to be cleared */
			/* 8 bytes because: host(6) + port(2) */
			send_fury_command(fdev, FCMD_CLEAR_PKTCACHE, (BYTE *)&pktcache_search.address.host, 8);
			/* start clear_req counter */
			pktcache_entry->clear_req = PKTCACHE_CLEAR_REQ_START;
			/* done */
			return(0);
		}
		
		/* if we are using length caching, we can not test anything */
		if( length_cached ) {
			/* adjust the ipx header length here! */
			new_ipx_header->length	= htons(pktcache_entry->data_length + sizeof(IPXHEADER));
		} else {
			/* test to make sure stuff is valid */
			/* does the data size match? */
			if( (ntohs(new_ipx_header->length) - sizeof(IPXHEADER)) != pktcache_entry->data_length ) {
				/* wrong data! -- send clear req if needed */
				if( pktcache_entry->clear_req <= 0 ) {
					errormsg(0, "proc_ipx", "cache data len (%d) mismatch! hdr (%d) (clear sent!)", pktcache_entry->data_length, ntohs(new_ipx_header->length));			
					/* tell the peer their data cache is bad */
					/* 8 bytes: host(6) + port(2) */
					send_fury_command(fdev, FCMD_CLEAR_PKTCACHE, (BYTE *)&pktcache_search.address.host, 8);
					
					/* adjust our cache entry so it will always fail */
					pktcache_entry->data_length = 0; 
					
					/* set the pktcache clear req */
					pktcache_entry->clear_req = PKTCACHE_CLEAR_REQ_START;
				} else {
					/* dec the clear req.. */
					pktcache_entry->clear_req--;
				}
				/* done */
				return(0);
			}
		}
		
		/* copy the data from the cache */
		memcpy(new_ipx_data, pktcache_entry->data, pktcache_entry->data_length);

		/* add to the used count */
		pktcache_entry->used++;

		/* adjust calculated length field */
		new_ipx_length += pktcache_entry->data_length;
		
	} else {
		/* the data for the ipx frame was sent to us	*/
		/* the current src pointer points to the 	*/
		/* data						*/
		memcpy(new_ipx_data, current_src, bytes_left);
		
		/* we need to cache this! so insert a src:port if it's not there */
		if( pktcache_entry == NULL ) {
			/* finish the search structure */
			pktcache_search.data_length	= bytes_left;
			pktcache_search.used 		= 0;
			pktcache_search.clear_req	= 0;
			memcpy(pktcache_search.data, current_src, bytes_left);
			
			/* insert this! */
			pktcache_entry = hash_insert(fdev->cache_ptrs.recv_pktcache, &pktcache_search);
			if( pktcache_entry == NULL ) {
				/* no hope! */
				errormsg(0, "proc_ipx", "could not insert new packet cache entry");
				return(0);
			}
		} else {
			/* modify existing src:port entry */
			pktcache_entry->data_length 	= bytes_left;
			pktcache_entry->used		= 0;
			memcpy(pktcache_entry->data, current_src, bytes_left);
		}
		
		/* adjust */
		new_ipx_length	+= pktcache_entry->data_length;
		bytes_left	 = 0;
	}
		
	/* -- now we should have a full IPX frame + data	-- */
	/* -- time to check it to make sure it is valid and if	-- */
	/* -- so send it out the bridge interface		-- */
	
	/* make sure we built a legal frame */
	if( (new_ipx_length < 30) || (new_ipx_length > FURY_IPX_FRAME_MAX) ) {
		errormsg(0, "process_ipx", "built ipx frame length error (%d bytes)", new_ipx_length);
		return(0);
	}
	
	/* check the length of the frame we built vs what the	*/
	/* header actually says it should be			*/
	if( new_ipx_length != ntohs(new_ipx_header->length) ) {
		errormsg(0, "process_ipx", "header length (%d) mismatch with built (%d)", ntohs(new_ipx_header->length), new_ipx_length);
		return(0);
	}
	
#if 0
	notice("Built IPX Frame:\n");
	hexdump(new_ipx_frame, new_ipx_length);
#endif
	
	/* now lookup the source and destination to determine where	*/
	/* this packet needs to go					*/
	if( ! get_address_locations(fdev, new_ipx_header, &hwaddr_src, &hwaddr_dest) ) {
		return(0);
	}
	
	/* do any post-processing -- only send if this says 'ok' */
	if( fdev->ipxframe_processor(fdev, new_ipx_header) ) {
	
		/* is the destination NULL (meaning broadcast or unknown) */
		if( hwaddr_dest->interface == NULL ) {
			/* send it to all interfaces */
			for( i = 0; i < ifg.total_interfaces; i++ ) {
				/* this is not us, so send it */
				ifg.interfaces[i].queue_ipx_frame(fdev, &ifg.interfaces[i], new_ipx_frame);
			} 		
		} else { 
			/* send it out the proper interface */
			hwaddr_dest->interface->queue_ipx_frame(fdev, hwaddr_dest->interface, new_ipx_frame);
		}
	}
		
		
	/* done */
	return(1);

}


/*************************************************************************
***********       IPX DATA HANDLER FUNCTIONS                     *********
*************************************************************************/

/*
**	GET_ADDRESS_LOCATIONS - returns a source and destination entry
*/
int get_address_locations(INTERFACE *iface, IPXHEADER *ipxheader, HWADDR_ENTRY **src_address, HWADDR_ENTRY **dest_address)
{
	HWADDR_ENTRY	search, *source, *dest;

	/* set the pointers to NULL */
	*src_address 	= NULL;
	*dest_address	= NULL;	
	
	/* build a search for the source address */
	
	/* put the source address in the search structure */
	memcpy(&search.hwaddr, &ipxheader->ipx_src.host, 6);
	
	/* retrieve it from the hardware address table */
	source = hash_get(&ifg.hwaddr_table, &search);
	
	/* if it does not exist, add it */
	if( source == NULL ) {
		/* set the interface to this one */
		search.interface = iface;
		
		/* add it to the table */
		source = hash_insert(&ifg.hwaddr_table, &search);
		if( source == NULL ) {	
			errormsg(0, "get_addr_loc", "could not add source address to hwaddr table!");
			return(0);
		} else {
			notice("New address discovered  %s -> %s\n", print_ether_addr(source->hwaddr), iface->name);
		}
	} else {
		/* make sure we set the interface to this one */
		if( source->interface == NULL ) {
			source->interface = iface;
			notice("New address discovered  %s -> %s\n", print_ether_addr(source->hwaddr), iface->name);
		}
	}
	
	/* put the destination in the search block */
	memcpy(&search.hwaddr, &ipxheader->ipx_dest.host, 6);
	
	/* try to locate the destination in the table */
	dest = hash_get(&ifg.hwaddr_table, &search);
	
	/* if it is not in the table, add it with location UNKNOWN */
	if( dest == NULL ) {
		/* set interface to UNKNOWN */
		search.interface = NULL;
		
		/* try to add it */
		dest = hash_insert(&ifg.hwaddr_table, &search);
		if( dest == NULL ) {
			errormsg(0, "get_addr_loc", "could not add destination hwaddr to table!");
			return(0);
		} 
	}
	
	/* set the ptrs */
	*src_address 	= source;
	*dest_address	= dest;
	
	/* did fine */
	return(1);

}




/****************************************************************************
**********                CACHE CONTROL FUNCTIONS                   *********
****************************************************************************/

/*
**	PROCESS_PKTCACHE_CLEAR
*/
int process_pktcache_clear(INTERFACE *fdev, BYTE *frame_data, int data_length)
{
	IPXADDR		pktcache_search;
	PKTCACHE_ENTRY	*entry;
	
	/* is the data valid? */
	if( data_length != 8 ) {
		errormsg(0, "proc_pktcache_clear", "bad packet size (%d bytes)", data_length);
		return(0);
	}
	
	/* clean out the search struct */
	memset(&pktcache_search, 0, sizeof(IPXADDR));
	
	/* create our search entry */
	memcpy(&pktcache_search.host, frame_data, 8);	/* fills in address and port */
	
	/* look it up */
	entry = hash_get(fdev->cache_ptrs.send_pktcache, &pktcache_search);
	if( entry == NULL ) {
		errormsg(0, "proc_pktcache_clear", "could not find source address:port combo!");
		return(0);
	}
	
	/* "clear" this entry by setting the data length to zero */
	entry->data_length = 0;

	/* little notice */
	notice("[%s] peer requested a data cache entry be cleared.\n", fdev->name);
	
	/* done */
	return(1);
	
}



/*
**	PROCESS_HDRCACHE_CLEAR
*/
int process_hdrcache_clear(INTERFACE *fdev, BYTE *frame_data, int data_length)
{
	BYTE		index;
	
	/* make sure length is good */
	if( data_length != 1 ) {
		errormsg(0, "proc_hdrcache_clear", "bad packet size (%d bytes)", data_length);
		return(0);
	}
	
	/* get index */
	index = *frame_data;

	/* is the index valid? */
	if( index >= fdev->cache_ptrs.send_hdrcache->size) {
		errormsg(0, "proc_cache_clear", "bad cache index (%d)", index);
		return(0);
	}
	
	/* clean out the entry */
	memset(&fdev->cache_ptrs.send_hdrcache->table[index], 0, sizeof(HDRCACHE_ENTRY));

	/* little notice */
	notice("[%s] peer requested header cache entry %d be cleared.\n", fdev->name, index);
	
	/* all done */
	return(1);
}


